------------------------------------------------------------
revno: 12572
revision-id: squid3@treenet.co.nz-20130605130009-5n1zi5oyf0j0dc1a
parent: squid3@treenet.co.nz-20130604065807-as5e7kodgjc70565
fixes bug(s): http://bugs.squid-cache.org/show_bug.cgi?id=3717
author: Alex Rousskov <rousskov@measurement-factory.com>
committer: Amos Jeffries <squid3@treenet.co.nz>
branch nick: 3.3
timestamp: Wed 2013-06-05 07:00:09 -0600
message:
  Bug 3717: assertion failed with dstdom_regex with IP based URL
  
  A combination of ACL negation and async lookup leads to
  Checklist.cc:287:"!needsAsync && !matchFinished" assertions.
  
  The lower-level ACL code says "not a match because I need an async lookup" but
  the negation-handling code in ACL::matches() ignores the "need an async
  lookup" part and converts "not a match" into a "match". This patch prevents
  that conversion, while allowing Checklist code to decide what to do with
  an async lookup (depending on whether the directive being checked supports
  slow ACLs).
  
  Note that this change prevents admins from negating async lookups in
  directives that do not support them: both "!foo" and "foo" will probably not
  match in those directives if ACL foo needs an async lookup.
------------------------------------------------------------
# Bazaar merge directive format 2 (Bazaar 0.90)
# revision_id: squid3@treenet.co.nz-20130605130009-5n1zi5oyf0j0dc1a
# target_branch: http://bzr.squid-cache.org/bzr/squid3/3.3
# testament_sha1: 92dad0f31f7594eaf986a709579b6c0160d92f2b
# timestamp: 2013-06-05 13:55:44 +0000
# source_branch: http://bzr.squid-cache.org/bzr/squid3/3.3
# base_revision_id: squid3@treenet.co.nz-20130604065807-\
#   as5e7kodgjc70565
# 
# Begin patch
=== modified file 'src/acl/Acl.cc'
--- src/acl/Acl.cc	2012-11-24 02:12:26 +0000
+++ src/acl/Acl.cc	2013-06-05 13:00:09 +0000
@@ -335,13 +335,24 @@
     AclMatchedName = _acl->name;
     debugs(28, 3, "ACLList::matches: checking " << (op ? null_string : "!") << _acl->name);
 
-    if (_acl->checklistMatches(checklist) != op) {
-        debugs(28, 4, "ACLList::matches: result is false");
-        return false;
+    bool result = false;
+    if (_acl->checklistMatches(checklist) == 1) {
+        debugs(28, 5, _acl->name << " matched" << (op ? "." : ", negating."));
+        result = (op != 0);
+    } else if (checklist->finished()) {
+        debugs(28, 5, _acl->name << " failed.");
+        result = false;
+    } else if (checklist->asyncNeeded()) {
+        debugs(28, 5, _acl->name << " needs async lookup");
+        result = false;
+    } else {
+        debugs(28, 5, _acl->name << " mismatched" << (op ? "." : ", negating."));
+        result = (op == 0);
     }
 
-    debugs(28, 4, "ACLList::matches: result is true");
-    return true;
+   debugs(28, 4, (op ? null_string : "!") << _acl->name << " result is " <<
+          (result ? "true" : "false"));
+   return result;
 }
 
 /*********************/

